package net.nosial.socialclient.classes;

import net.nosial.socialclient.exceptions.ResolutionException;
import net.nosial.socialclient.objects.ResolvedServer;

import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.Hashtable;

// Improved Resolver class
public class Resolver
{
    /**
     * Resolves the domain to obtain endpoint and public key from its DNS TXT records.
     *
     * @param domain The domain to be resolved.
     * @return ResolvedServer An instance of ResolvedServer containing the endpoint and public key.
     * @throws ResolutionException If the DNS TXT records cannot be resolved or if required information is missing.
     */
    public static ResolvedServer resolveDomain(String domain) throws ResolutionException
    {
        final String endpointPrefix = "socialbox=";
        final String keyPrefix = "socialbox-key=";

        // Disable caching
        Hashtable<String, String> env = new Hashtable<>();
        env.put("com.sun.jndi.dns.timeout.retries", "1");
        env.put("com.sun.jndi.dns.cache.ttl", "0");
        env.put("com.sun.jndi.dns.cache.negative.ttl", "0");

        try
        {
            DirContext dirContext = new InitialDirContext(env);
            Attributes attributes = dirContext.getAttributes("dns:/" + domain, new String[]{"TXT"});
            Attribute attributeTXT = attributes.get("TXT");

            if (attributeTXT == null)
            {
                throw new ResolutionException("Failed to resolve DNS TXT records for " + domain);
            }

            String endpoint = null;
            StringBuilder publicKeyBuilder = new StringBuilder();
            boolean publicKeyFound = false;

            for (int i = 0; i < attributeTXT.size(); i++)
            {
                String value = (String) attributeTXT.get(i);

                // Remove surrounding quotes if present
                if (value.startsWith("\"") && value.endsWith("\""))
                {
                    value = value.substring(1, value.length() - 1);
                }

                // Split the value into fragments to find the relevant keys
                String[] fragments = value.split("\\s+");
                for (String fragment : fragments)
                {
                    if (fragment.startsWith(endpointPrefix))
                    {
                        endpoint = fragment.substring(endpointPrefix.length());
                    }
                    else if (fragment.startsWith(keyPrefix))
                    {
                        publicKeyBuilder.append(fragment.substring(keyPrefix.length()));
                        publicKeyFound = true;
                    }
                    else if (publicKeyFound)
                    {
                        // If the public key has already started, append the fragment
                        publicKeyBuilder.append(fragment);
                    }
                }
            }

            if (endpoint == null)
            {
                throw new ResolutionException("Failed to resolve RPC endpoint for " + domain);
            }

            if (publicKeyBuilder.isEmpty())
            {
                throw new ResolutionException("Failed to resolve public key for " + domain);
            }

            String publicKey = publicKeyBuilder.toString();
            return new ResolvedServer(endpoint, publicKey);
        }
        catch (NamingException e)
        {
            throw new ResolutionException("Error resolving domain: " + e.getMessage(), e);
        }
    }
}